@@ -13,8 +13,12 @@
 #' \item \code{db = "ensembl"}
 #' \item \code{db = "ensemblgenomes"}
 #' } 
-#' @param organism a character string specifying the scientific name of the 
-#' organism of interest, e.g. \code{organism = "Homo sapiens"}.
+#' @param organism there are three options to characterize an organism: 
+#' \itemize{
+#' \item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
+#' \item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
+#' \item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
+#' }
 #' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.
 #' @param path a character string specifying the location (a folder) 
 #' in which the corresponding CDS file shall be stored. 
@@ -50,8 +54,14 @@ getCDS <-
                 call. = FALSE
             )
             
-        message("Starting CDS retrieval '", organism, "' from ", db, " ...")
-        message("\n")
+        if (db == "ensemblgenomes") {
+            organism_name <- is.genome.available(db = db, organism = organism, details = TRUE)$display_name[1]
+            message("Starting CDS retrieval of '", organism_name, "' from ", db, " ...")
+            message("\n")
+        } else {
+            message("Starting CDS retrieval of '", organism, "' from ", db, " ...")
+            message("\n")
+        }
         
         if (is.element(db, c("refseq", "genbank"))) {
             # get Kingdom Assembly Summary file
@@ -59,13 +69,13 @@ getCDS <-
                 getKingdomAssemblySummary(db = db)
             
             # test wheter or not genome is available
-            is.genome.available(organism = organism, db = db)
+            suppressMessages(is.genome.available(organism = organism, db = db))
             
             if (!file.exists(path)) {
                 dir.create(path, recursive = TRUE)
             }
             
-            organism_name <-
+            organism_name <- assembly_accession <- taxid <-
                 refseq_category <- version_status <- NULL
             organism <-
                 stringr::str_replace_all(organism, "\\(", "")
@@ -73,41 +83,65 @@ getCDS <-
                 stringr::str_replace_all(organism, "\\)", "")
             
             if (reference) {
+                if (!is.taxid(organism)) {
                     FoundOrganism <-
-                            dplyr::filter(
-                                    AssemblyFilesAllKingdoms,
-                                    stringr::str_detect(organism_name, organism),
-                                    ((refseq_category == "representative genome") |
-                                             (refseq_category == "reference genome")
-                                    ),
-                                    (version_status == "latest")
-                            ) 
+                        dplyr::filter(
+                            AssemblyFilesAllKingdoms,
+                            stringr::str_detect(organism_name, organism) | 
+                                stringr::str_detect(assembly_accession, organism),
+                            ((refseq_category == "representative genome") |
+                                 (refseq_category == "reference genome")
+                            ),
+                            (version_status == "latest")
+                        ) 
+                } else {
+                    FoundOrganism <-
+                        dplyr::filter(
+                            AssemblyFilesAllKingdoms,
+                            taxid == as.integer(organism),
+                            ((refseq_category == "representative genome") |
+                                 (refseq_category == "reference genome")
+                            ),
+                            (version_status == "latest"))
+                }
             } else {
+                if (!is.taxid(organism)) {
                     FoundOrganism <-
-                            dplyr::filter(
-                                    AssemblyFilesAllKingdoms,
-                                    stringr::str_detect(organism_name, organism),
-                                    (version_status == "latest")
-                            ) 
+                        dplyr::filter(
+                            AssemblyFilesAllKingdoms,
+                            stringr::str_detect(organism_name, organism) |
+                                stringr::str_detect(assembly_accession, organism),
+                            (version_status == "latest")
+                        ) 
+                } else {
+                    FoundOrganism <-
+                        dplyr::filter(
+                            AssemblyFilesAllKingdoms,
+                            taxid == as.integer(organism),
+                            (version_status == "latest")
+                        ) 
+                }
             }
             
+            
             if (nrow(FoundOrganism) == 0) {
-                    message(
-                            paste0(
-                                    "----------> No reference genome or representative genome was found for '",
-                                    organism, "'. Thus, download for this species has been omitted.",
-                                    " Have you tried to specify 'reference = FALSE' ?"
-                            )
+                message(
+                    paste0(
+                        "----------> No reference CDS or representative CDS was found for '",
+                        organism, "'. Thus, download for this organism has been omitted.",
+                        " Have you tried to specify getCDS(db = '",db,"', organism = '",organism,"' , reference = FALSE) ?",
+                        " Alternatively, you can retrieve CDS using the NCBI accession ID or NCBI Taxonomy ID.",
+                        " See '?'is.genome.available' for examples."
                     )
+                )
                     return("Not available")
             } else {
                 if (nrow(FoundOrganism) > 1) {
                     warnings(
                         "More than one entry has been found for '",
-                        organism,
-                        "'. Only the first entry '",
-                        FoundOrganism[1, 1],
-                        "' has been used for subsequent genomic CDS retrieval."
+                        organism, "'. Only the first entry '", FoundOrganism[1, 1], "' has been used for subsequent CDS retrieval.",
+                        " If you wish to download a different version, please use the NCBI accession ID when specifying the 'organism' argument.",
+                        " See ?is.genome.available for examples."
                     )
                     FoundOrganism <- FoundOrganism[1, ]
                 }
@@ -130,15 +164,15 @@ getCDS <-
                 local.org <-
                     stringr::str_replace_all(organism, "\\/", "_")
                 
-                if (!exists.ftp.file(url = paste0(FoundOrganism$ftp_path, "/"),
-                                     file.path = download_url)) {
-                    message(
-                    "Unfortunately no CDS file could be found for organism '",
-                        organism,
-                    "'. Thus, the download of this organism has been omitted."
-                    )
-                    return(FALSE)
-                }
+                # if (!exists.ftp.file(url = paste0(FoundOrganism$ftp_path, "/"),
+                #                      file.path = download_url)) {
+                #     message(
+                #     "Unfortunately no CDS file could be found for organism '",
+                #         organism,
+                #     "'. Thus, the download of this organism has been omitted."
+                #     )
+                #     return(FALSE)
+                # }
                 
                 if (nrow(FoundOrganism) == 1) {
                     if (file.exists(file.path(
@@ -259,6 +293,29 @@ getCDS <-
                         submitter = FoundOrganism$submitter
                     )
                     
+                    doc <- tibble::tibble(
+                        file_name = paste0(ifelse(is.taxid(organism), paste0("taxid_", local.org), local.org), "_genomic_", db,
+                                           ".fna.gz"),
+                        organism  = organism,
+                        url       = download_url,
+                        database  = db,
+                        path      = path,
+                        refseq_category = FoundOrganism$refseq_category,
+                        assembly_accession = FoundOrganism$assembly_accession,
+                        bioproject = FoundOrganism$bioproject,
+                        biosample = FoundOrganism$biosample,
+                        taxid = FoundOrganism$taxid,
+                        infraspecific_name = FoundOrganism$infraspecific_name,
+                        version_status = FoundOrganism$version_status,
+                        release_type = FoundOrganism$release_type,
+                        genome_rep = FoundOrganism$genome_rep,
+                        seq_rel_date = FoundOrganism$seq_rel_date,
+                        submitter = FoundOrganism$submitter
+                        
+                    )
+                    
+                    readr::write_tsv(doc, path = file.path(path,paste0("doc_",local.org,"_db_",db,".tsv")))
+                    
                     message(
                         paste0(
                             "The genomic CDS of '",
@@ -308,28 +365,60 @@ getCDS <-
                 getENSEMBL.Seq(organism, type = "cds", id.type = "all", path)
             
             if (is.logical(cds.path)) {
-                invisible(return(TRUE))
+                if (!cds.path)
+                    return(FALSE)
             } else {
-                new.organism <- stringr::str_replace_all(organism, " ", "_")
                 
-                # test proper API access
-                tryCatch({
-                    json.qry.info <-
-                        jsonlite::fromJSON(
-                            paste0(
-                                "http://rest.ensembl.org/info/assembly/",
-                                new.organism,
-                                "?content-type=application/json"
-                            )
-                        )
-                }, error = function(e)
-                    stop(
-                    "The API 'http://rest.ensembl.org' does not seem to work
-                    properly. Are you connected to the internet? Is the 
-                    homepage 'http://rest.ensembl.org' currently available?",
-                        call. = FALSE
+                taxon_id <- assembly <- name <- accession <- NULL
+                
+                ensembl_summary <-
+                    suppressMessages(is.genome.available(
+                        organism = organism,
+                        db = "ensembl",
+                        details = TRUE
                     ))
                 
+                if (nrow(ensembl_summary) > 1) {
+                    if (is.taxid(organism)) {
+                        ensembl_summary <-
+                            dplyr::filter(ensembl_summary, taxon_id == as.integer(organism), !is.na(assembly))
+                    } else {
+                        
+                        ensembl_summary <-
+                            dplyr::filter(
+                                ensembl_summary,
+                                (name == stringr::str_to_lower(stringr::str_replace_all(organism, " ", "_"))) |
+                                    (accession == organism),
+                                    !is.na(assembly)
+                            )
+                    }
+                }
+                
+                new.organism <- ensembl_summary$name[1]
+                new.organism <-
+                    paste0(
+                        stringr::str_to_upper(stringr::str_sub(new.organism, 1, 1)),
+                        stringr::str_sub(new.organism, 2, nchar(new.organism))
+                    )     
+                
+                url_api <- paste0(
+                    "http://rest.ensembl.org/info/assembly/",
+                    new.organism,
+                    "?content-type=application/json"
+                )
+                
+                # choose only first entry if not specified otherwise
+                if (length(url_api) > 1)
+                    url_api <- url_api[1]
+                
+                if (curl::curl_fetch_memory(url_api)$status_code != 200) {
+                    message("The API call '",url_api,"' did not work. This might be due to a non-existing organism that you specified or a corrupted internet or firewall connection.")
+                    return("Not available")
+                }
+                
+                # retrieve information from API
+                json.qry.info <- jsonlite::fromJSON(url_api)
+                
                 # generate CDS documentation
                 sink(file.path(
                     path,
@@ -369,6 +458,24 @@ getCDS <-
                 
                 sink()
                 
+                doc <- tibble::tibble(
+                    file_name = cds.path,
+                    organism = new.organism,
+                    database = db,
+                    download_data = date(),
+                    assembly_name = ifelse(!is.null(json.qry.info$assembly_name), json.qry.info$assembly_name, "none"),
+                    assembly_date = ifelse(!is.null(json.qry.info$assembly_date), json.qry.info$assembly_date, "none"),
+                    genebuild_last_geneset_update = ifelse(!is.null(json.qry.info$genebuild_last_geneset_update), json.qry.info$genebuild_last_geneset_update, "none"), 
+                    assembly_accession = ifelse(!is.null(json.qry.info$assembly_accession), json.qry.info$assembly_accession, "none"),
+                    genebuild_initial_release_date = ifelse(!is.null(json.qry.info$genebuild_initial_release_date), json.qry.info$genebuild_initial_release_date, "none")
+                    
+                )
+                
+                readr::write_tsv(doc, file.path(
+                    path,
+                    paste0("doc_", new.organism, "_db_", db, ".tsv"))
+                )
+                
                 message(
                     paste0(
                         "The CDS of '",
@@ -397,28 +504,59 @@ getCDS <-
                                       id.type = "all", path)
             
             if (is.logical(cds.path)) {
-                invisible(return(TRUE))
+                if (!cds.path)
+                    return(FALSE)
             } else {
-                new.organism <- stringr::str_replace_all(organism, " ", "_")
+                
+                taxon_id <- assembly <- name <- accession <- NULL
+                
+                ensembl_summary <-
+                    suppressMessages(is.genome.available(
+                        organism = organism,
+                        db = "ensemblgenomes",
+                        details = TRUE
+                    ))
+                
+                if (nrow(ensembl_summary) > 1) {
+                    if (is.taxid(organism)) {
+                        ensembl_summary <-
+                            dplyr::filter(ensembl_summary, taxon_id == as.integer(organism), !is.na(assembly))
+                    } else {
+                        
+                        ensembl_summary <-
+                            dplyr::filter(
+                                ensembl_summary,
+                                (name == stringr::str_to_lower(stringr::str_replace_all(organism, " ", "_"))) |
+                                    (accession == organism),
+                                    !is.na(assembly)
+                            )
+                    }
+                }
+                
+                new.organism <- ensembl_summary$name[1]
+                new.organism <-
+                    paste0(
+                        stringr::str_to_upper(stringr::str_sub(new.organism, 1, 1)),
+                        stringr::str_sub(new.organism, 2, nchar(new.organism))
+                    ) 
+                
+                rest_url <- paste0(
+                    "http://rest.ensemblgenomes.org/info/assembly/",
+                    new.organism,
+                    "?content-type=application/json"
+                )
+                
+                if (curl::curl_fetch_memory(rest_url)$status_code != 200) {
+                    warning(
+                        "The url: '",rest_url,"' cannot be reached. This might be due to a connection issue or incorrect url path (e.g. not valid organism name).",
+                        call. = FALSE)
+                    return(FALSE)
+                }
                 
                 # test proper API access
-                tryCatch({
-                    json.qry.info <-
-                        jsonlite::fromJSON(
-                            paste0(
-                                "http://rest.ensemblgenomes.org/info/assembly/",
-                                new.organism,
-                                "?content-type=application/json"
-                            )
-                        )
-                }, error = function(e)
-                    stop(
-                        "The API 'http://rest.ensemblgenomes.org' does not seem 
-                        to work properly. Are you connected to the internet? 
-                        Is the homepage 'http://rest.ensemblgenomes.org' 
-                        currently available?",
-                        call. = FALSE
-                    ))
+                json.qry.info <-
+                    jsonlite::fromJSON(rest_url)
+                
                 
                 # generate CDS documentation
                 sink(file.path(
@@ -459,6 +597,24 @@ getCDS <-
                 
                 sink()
                 
+                doc <- tibble::tibble(
+                    file_name = cds.path,
+                    organism = new.organism,
+                    database = db,
+                    download_data = date(),
+                    assembly_name = ifelse(!is.null(json.qry.info$assembly_name), json.qry.info$assembly_name, "none"),
+                    assembly_date = ifelse(!is.null(json.qry.info$assembly_date), json.qry.info$assembly_date, "none"),
+                    genebuild_last_geneset_update = ifelse(!is.null(json.qry.info$genebuild_last_geneset_update), json.qry.info$genebuild_last_geneset_update, "none"),
+                    assembly_accession = ifelse(!is.null(json.qry.info$assembly_accession), json.qry.info$assembly_accession, "none"), 
+                    genebuild_initial_release_date = ifelse(!is.null(json.qry.info$genebuild_initial_release_date), json.qry.info$genebuild_initial_release_date, "none")
+                    
+                )
+                
+                readr::write_tsv(doc, file.path(
+                    path,
+                    paste0("doc_", new.organism, "_db_", db, ".tsv"))
+                )
+                
                 message(
                     paste0(
                         "The CDS of '",
