@@ -5,39 +5,79 @@ knitr::opts_chunk$set(
   error = FALSE,
   tidy = FALSE)
 
+## ----eval=TRUE--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+# install the biomaRt package		
+# source("http://bioconductor.org/biocLite.R")		
+# biocLite("biomaRt")		
+# load biomaRt		
+library(biomaRt)		
+# look at top 10 databases		
+head(biomaRt::listMarts(host = "www.ensembl.org"), 10)		
+
+## ----eval=TRUE--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+head(biomaRt::listDatasets(biomaRt::useMart("ENSEMBL_MART_ENSEMBL", host = "www.ensembl.org")), 10)		
+
+## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+#  head(biomaRt::listAttributes(biomaRt::useDataset(
+#                                           dataset = "hsapiens_gene_ensembl", 		
+#                                           mart    = useMart("ENSEMBL_MART_ENSEMBL",		
+#                                           host    = "www.ensembl.org"))), 10)		
+
+## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+#   head(biomaRt::listFilters(biomaRt::useDataset(dataset = "hsapiens_gene_ensembl", 		
+#                                                 mart    = useMart("ENSEMBL_MART_ENSEMBL",		
+#                                                 host    = "www.ensembl.org"))), 10)		
+
+## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+#   # 1) select a mart and data set		
+#   mart <- biomaRt::useDataset(dataset = "hsapiens_gene_ensembl", 		
+#                      mart    = useMart("ENSEMBL_MART_ENSEMBL",		
+#                      host    = "www.ensembl.org"))		
+#   		
+#   # 2) run a biomart query using the getBM() function		
+#   # and specify the attributes and filter arguments		
+#   geneSet <- "GUCA2A"		
+#   		
+#   resultTable <- biomaRt::getBM(attributes = c("start_position","end_position","description"),		
+#                        filters    = "hgnc_symbol", 		
+#                        values     = geneSet, 		
+#                        mart       = mart)		
+#   		
+#   resultTable 		
+
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # load the biomartr package
 #  library(biomartr)
 #  
 #  # list all available databases
-#  getMarts()
+#  biomartr::getMarts()
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-#  head(getDatasets(mart = "ENSEMBL_MART_ENSEMBL") , 5)
+#  head(biomartr::getDatasets(mart = "ENSEMBL_MART_ENSEMBL") , 5)
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-#  tail(getDatasets(mart = "ENSEMBL_MART_ENSEMBL") , 38)
+#  tail(biomartr::getDatasets(mart = "ENSEMBL_MART_ENSEMBL") , 38)
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # list all available attributes for dataset: hsapiens_gene_ensembl
-#  head( getAttributes(mart    = "ENSEMBL_MART_ENSEMBL",
-#                      dataset = "hsapiens_gene_ensembl"), 10 )
+#  head( biomartr::getAttributes(mart    = "ENSEMBL_MART_ENSEMBL",
+#                                dataset = "hsapiens_gene_ensembl"), 10 )
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # list all available filters for dataset: hsapiens_gene_ensembl
-#  head( getFilters(mart    = "ENSEMBL_MART_ENSEMBL",
-#                   dataset = "hsapiens_gene_ensembl"), 10 )
+#  head( biomartr::getFilters(mart    = "ENSEMBL_MART_ENSEMBL",
+#                             dataset = "hsapiens_gene_ensembl"), 10 )
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # retrieving all available datasets and biomart connections for
 #  # a specific query organism (scientific name)
-#  organismBM(organism = "Homo sapiens")
+#  biomartr::organismBM(organism = "Homo sapiens")
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
@@ -49,62 +89,62 @@ knitr::opts_chunk$set(
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # search for attribute topic "id"
-#  head(organismAttributes("Homo sapiens", topic = "id"), 20)
+#  head(biomartr::organismAttributes("Homo sapiens", topic = "id"), 20)
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # search for attribute topic "homolog"
-#  head(organismAttributes("Homo sapiens", topic = "homolog"), 20)
+#  head(biomartr::organismAttributes("Homo sapiens", topic = "homolog"), 20)
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # search for attribute topic "dn"
-#  head(organismAttributes("Homo sapiens", topic = "dn"))
+#  head(biomartr::organismAttributes("Homo sapiens", topic = "dn"))
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # search for attribute topic "ds"
-#  head(organismAttributes("Homo sapiens", topic = "ds"))
+#  head(biomartr::organismAttributes("Homo sapiens", topic = "ds"))
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # return available filters for "Homo sapiens"
-#  head(organismFilters("Homo sapiens"), 20)
+#  head(biomartr::organismFilters("Homo sapiens"), 20)
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # search for filter topic "id"
-#  head(organismFilters("Homo sapiens", topic = "id"), 20)
+#  head(biomartr::organismFilters("Homo sapiens", topic = "id"), 20)
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  
-#  head(organismAttributes("Homo sapiens", topic = "id"))
+#  head(biomartr::organismAttributes("Homo sapiens", topic = "id"))
 
 ## ----eval=FALSE-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # retrieve the proteome of Homo sapiens from refseq
-#  file_path <- getProteome( db       = "refseq",
-#                            organism = "Homo sapiens",
-#                            path     = file.path("_ncbi_downloads","proteomes") )
+#  file_path <- biomartr::getProteome( db       = "refseq",
+#                                      organism = "Homo sapiens",
+#                                      path     = file.path("_ncbi_downloads","proteomes") )
 #  
-#  Hsapiens_proteome <- read_proteome(file_path, format = "fasta")
+#  Hsapiens_proteome <- biomartr::read_proteome(file_path, format = "fasta")
 #  
 #  # remove splice variants from id
 #  gene_set <- unlist(sapply(strsplit(Hsapiens_proteome@ranges@NAMES[1:5], ".",fixed = TRUE), function(x) x[1]))
 #  
-#  result_BM <- biomart( genes      = gene_set,
-#                        mart       = "ENSEMBL_MART_ENSEMBL",
-#                        dataset    = "hsapiens_gene_ensembl",
-#                        attributes = c("ensembl_gene_id","ensembl_peptide_id"),
-#                        filters    = "refseq_peptide")
+#  result_BM <- biomartr::biomart( genes      = gene_set, # genes were retrieved using biomartr::getGenome()
+#                                  mart       = "ENSEMBL_MART_ENSEMBL", # marts were selected with biomartr::getMarts()
+#                                  dataset    = "hsapiens_gene_ensembl", # datasets were selected with biomartr::getDatasets()
+#                                  attributes = c("ensembl_gene_id","ensembl_peptide_id"), # attributes were selected with biomartr::getAttributes()
+#                                  filters    = "refseq_peptide") # specify what ID type was stored in the fasta file retrieved with biomartr::getGenome()
 #  
 #  result_BM
 
@@ -112,9 +152,9 @@ knitr::opts_chunk$set(
 #  # show all elements of the data.frame
 #  options(tibble.print_max = Inf)
 #  # search for GO terms of an example Homo sapiens gene
-#  GO_tbl <- getGO(organism = "Homo sapiens",
-#                  genes    = "GUCA2A",
-#                  filters  = "hgnc_symbol")
+#  GO_tbl <- biomartr::getGO(organism = "Homo sapiens",
+#                            genes    = "GUCA2A",
+#                            filters  = "hgnc_symbol")
 #  
 #  GO_tbl
 
